#property copyright "Copyright 2023, Anil Varma"
#property link      "AnilHVarma@gmail.com"
#property version   "1.00"

#include "../../Includes/Base/BaseIndicator.mqh"
//#define  iZigZagPath "Indicators\\BNB\\Trend\\iZigZag.ex5"
//#resource	"\\" + iZigZagPath

// CUSTOM INDICATOR BUFFER NUMBERS SPECIFIED WITH #define MACRO SUBSTITUTION
#define bufferMajorLO 0
#define bufferMajorHI 1
#define bufferMinorLO 2
#define bufferMinorHI 3
//+-----------------------------------------------------------------------------------------------------------------------------+
//| CLASS:        CiZigZag.mqh
//| APPLICATION:  RETURNS CUSTOM INDICATOR ZigZag.mq5 VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
class CiZigZag final : public CBaseIndicator {

public:

	// DEFAULT EMPTY CONSTRUCTOR TO SET CLASS WITH CURRENT SYMBOL AND CHART. NOTE: Init() METHOD IS NOT CALLED
	  CiZigZag() : CBaseIndicator(_Symbol,PERIOD_CURRENT) { }

	// NOW USE 'new' OPERATOR FOR PARAMETRIC CONSTRUCTOR TO SET CORRECT PARAMETERS
	  CiZigZag(string pSymbol,ENUM_TIMEFRAMES pTimeFrame,int pZZSlowPeriod,int pZZFastPeriod)
	  	: CBaseIndicator(pSymbol,pTimeFrame) {
	  		Init(pZZSlowPeriod,pZZFastPeriod);
	  	}
	 ~CiZigZag() { IndicatorRelease(mHandleZZ); }

	// void USED INSTEAD OF int TO REINITIALISATION OF HANDLE IF INVALID AT ANY POINT OF TIME
	// RESTRIC CLASS INITIALIAZATION BY PARAMETRIC CONSTRUCTOR ONLY TO MAKE SURE WE HAVE CORRECT Symbol AND TimeFrame
private:
	  void      Init(int pZZSlowPeriod,int pZZFastPeriod);

private:

	// MEMBER VARIABLES
		int					mHandleZZ;
	  int        	mSlowPeriod;			// ZIG ZAG SLOW PERIOD (MAJOR SWING)
	  int        	mFastPeriod;			// ZIG ZAG FAST PERIOD (MINOR SWING)
	  int        	mBarStart;
	  int        	mLookBackMax;     // lookback Maximum No of Bars to find High/Low Index/Value
	  int        	mMaxIndex;        // lookback Major/Minor No of Index/Values
	  int					mSize;            // Array size

  // MEMBER METHODS FOR INTERNAL CALCULATIONS
	  void        GetMajorLO(SZigZag &pMajorLO[]);
	  void        GetMajorHI(SZigZag &pMajorHI[]);
	  void        GetMinorLO(SZigZag &pMinorLO[]);
	  void        GetMinorHI(SZigZag &pMinorHI[]);
	  double      GetIndexZZ(int pBufferNo,int pIndex);

public:

	// METHODS TO RETURN INDICATOR VALUES TO CALLING EA / CLASS
		void 				GetArrayZZAll(SZigZag &pMajorLO[],SZigZag &pMajorHI[],SZigZag &pMinorLO[],SZigZag &pMinorHI[]);
		void 				GetArrayMajor(SZigZag &pMajorLO[],SZigZag &pMajorHI[]);
		void 				GetArrayMinor(SZigZag &pMinorLO[],SZigZag &pMinorHI[]);

}; // END Of Class Definition
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:				Init()
//| APPLICATION:	METHOD TO INITIALIAZE MEMBER VARIABLES OF THE INDICATOR
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::Init(int pZZSlowPeriod,int pZZFastPeriod) {

		// ASSIGN OR SET ZigZag INDICATOR PARAMETERS
	  mSlowPeriod  = pZZSlowPeriod;
	  mFastPeriod  = pZZFastPeriod;
	  mBarStart    = 0;               // SET AT 'Zero' TO ENABLE RETURNING ACTUAL BAR INDEX OF CHART WITH CounterHI/LO
	  mLookBackMax = 500;           	// MAXIMUM BARS TO LOOK BACK
	  mMaxIndex    = 5;       				// LOOK FOR MAXIMUM OF 'n' ZigZag HIGH / LOW VALUES
	  mSize        = mMaxIndex + 1;   // ALLOW ONE EXTRA ARRAY ELEMENT SPACE FOR INDEX[0]

		// DECLARE AND CHECK VALIDITY ON HANDLE
	  TesterHideIndicators(true);
	  //mHandleZZ = iCustom(mSymbol,mTimeFrame,"::" + iZigZagPath,mSlowPeriod,mFastPeriod);
	  mHandleZZ = iCustom(mSymbol,mTimeFrame,"Indicators\\BNB\\Trend\\iZigZag",mSlowPeriod,mFastPeriod);
	  if(mHandleZZ == INVALID_HANDLE) {
			ResetLastError();
	    PrintFormat("%s: Error[%i] creating handle for iZigZag",string(__FUNCTION__),GetLastError());
	  }
	  TesterHideIndicators(false);

		//return(INIT_SUCCEEDED);

} // END Of method Init()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetArrayZZAll()
//| APPLICATION:  METHOD TO GET MAJOR AND MINOR / HIGH AND LOW ZigZag VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetArrayZZAll(SZigZag &pMajorLO[],SZigZag &pMajorHI[],SZigZag &pMinorLO[],SZigZag &pMinorHI[]) {

	  GetMajorLO(pMajorLO);
	  GetMajorHI(pMajorHI);

	  GetMinorLO(pMinorLO);
	  GetMinorHI(pMinorHI);

} // END Of method GetArrayZZAll()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetArrayMajor()
//| APPLICATION:  METHOD TO GET MAJOR HIGH AND LOW ZigZag VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetArrayMajor(SZigZag &pMajorLO[],SZigZag &pMajorHI[]) {

	  GetMajorLO(pMajorLO);
	  GetMajorHI(pMajorHI);

} // END Of method GetArrayMajor()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetArrayMinor()
//| APPLICATION:  METHOD TO GET MINOR HIGH AND LOW ZigZag VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetArrayMinor(SZigZag &pMinorLO[],SZigZag &pMinorHI[]) {

	  GetMinorLO(pMinorLO);
	  GetMinorHI(pMinorHI);

} // END Of method GetArrayZZAll()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetMajorLO()
//| APPLICATION:  METHOD TO RETURN ZigZag MajorLO VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetMajorLO(SZigZag &pMajorLO[]) {

		// CHECK HANDLE IS VALID, ELSE INITIALZE AGAIN
	  if(mHandleZZ == INVALID_HANDLE) 																			Init(mFastPeriod,mSlowPeriod);

		// PREPARE THE PASSED IN ARRAY
		ArraySetAsSeries(pMajorLO,true);
		ArrayResize(pMajorLO,mSize);

		// ARRAY INDEX[0] INTIALIZED WITH NULL VALUES TO MATCH MajorLO[1] WITH ARRAY INDEX[1]
		pMajorLO[0].idx 	= 0;
		pMajorLO[0].price = 0.00;

		// LOOP UNTIL FOUND REQUIRED NUMBER OF MajorLO VALUES
		int 	 count 	 = 0;
		double priceZZ = 0.00;

		for(int i = mBarStart; i < mLookBackMax; i++) {

	    priceZZ = GetIndexZZ(bufferMajorLO,i);															// GET ZigZag[i] VALUE FOR bufferMajorLO
			if(priceZZ <= 0.00)			continue;																		// CONTINUE TO NEXT INDEX IN LOOP

	    if(count <= mMaxIndex) {    																				// ADD VALUE TO pMajorLO ARRAY
	      pMajorLO[count+1].idx 	= i;																			// ON FIRST MajorLO, count = 0 + 1 FOR INDEX[1]
	      pMajorLO[count+1].price = priceZZ;
	      count++;
	    }

	    if(count >= mMaxIndex) 	return;           													// EXIT, IF REQUIRED mMaxIndex VALUES ARE FOUND
	  }

} // END Of method GetMajorLO()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetMajorHI()
//| APPLICATION:  METHOD TO RETURN ZigZag MajorHI VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetMajorHI(SZigZag &pMajorHI[]) {

		// CHECK HANDLE IS VALID, ELSE INITIALZE AGAIN
	  if(mHandleZZ == INVALID_HANDLE) 																			Init(mFastPeriod,mSlowPeriod);

		// PREPARE THE PASSED IN ARRAY
		ArraySetAsSeries(pMajorHI,true);
		ArrayResize(pMajorHI,mSize);

		// ARRAY INDEX[0] INTIALIZED WITH NULL VALUES TO MATCH MajorHI[1] WITH ARRAY INDEX[1]
		pMajorHI[0].idx 	= 0;
		pMajorHI[0].price = 0.00;

		// LOOP UNTIL FOUND REQUIRED NUMBER OF MajorHI VALUES
		int 	 count 	 = 0;
		double priceZZ = 0.00;

	  for(int i = mBarStart; i < mLookBackMax; i++) {

	    priceZZ = GetIndexZZ(bufferMajorHI,i);															// GET ZigZag[i] VALUE FOR bufferMajorHI
			if(priceZZ <= 0.00)			continue;																		// CONTINUE TO NEXT INDEX IN LOOP

	    if(count <= mMaxIndex) {    																				// ADD VALUE TO pMajorLO ARRAY
	      pMajorHI[count+1].idx 	= i;																			// ON FIRST MajorHI, count = 0 + 1 FOR INDEX[1]
	      pMajorHI[count+1].price = priceZZ;
	      count++;
	    }

    	if(count >= mMaxIndex)   return;           													// EXIT, IF REQUIRED mMaxIndex VALUES ARE FOUND
  	}

} // END Of method GetMajorHI()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetMinorLO()
//| APPLICATION:  METHOD TO RETURN ZigZag MinorLO VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetMinorLO(SZigZag &pMinorLO[]) {

		// CHECK HANDLE IS VALID, ELSE INITIALZE AGAIN
	  if(mHandleZZ == INVALID_HANDLE) 																			Init(mFastPeriod,mSlowPeriod);

		// PREPARE THE PASSED IN ARRAY
		ArraySetAsSeries(pMinorLO,true);
		ArrayResize(pMinorLO,mSize);

		// ARRAY INDEX[0] INTIALIZED WITH NULL VALUES TO MATCH MinorLO[1] WITH ARRAY INDEX[1]
		pMinorLO[0].idx 	= 0;
		pMinorLO[0].price = 0.00;

		// LOOP UNTIL FOUND REQUIRED NUMBER OF MinorLO VALUES
		int 	 count 	 = 0;
		double priceZZ = 0.00;

	  for(int i = mBarStart; i < mLookBackMax; i++) {

	    priceZZ = GetIndexZZ(bufferMinorLO,i);															// GET ZigZag[i] VALUE FOR bufferMinorLO
			if(priceZZ <= 0.00)			continue;																		// CONTINUE TO NEXT INDEX IN LOOP

	    if(count <= mMaxIndex) {    																				// ADD VALUE TO pMinorLO ARRAY
	      pMinorLO[count+1].idx 	= i;																			// ON FIRST MinorLO, count = 0 + 1 FOR INDEX[1]
	      pMinorLO[count+1].price = priceZZ;
	      count++;
	    }

	    if(count >= mMaxIndex)   return;           													// EXIT, IF REQUIRED mMaxIndex VALUES ARE FOUND
	  }

} // END Of method GetMinorLO()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:       GetMinorHI()
//| APPLICATION:  METHOD TO RETURN ZigZag MinorHI VALUES
//+-----------------------------------------------------------------------------------------------------------------------------+
void CiZigZag::GetMinorHI(SZigZag &pMinorHI[]) {

		// CHECK HANDLE IS VALID, ELSE INITIALZE AGAIN
	  if(mHandleZZ == INVALID_HANDLE) 																			Init(mFastPeriod,mSlowPeriod);

		// PREPARE THE PASSED IN ARRAY
		ArraySetAsSeries(pMinorHI,true);
		ArrayResize(pMinorHI,mSize);

		// ARRAY INDEX[0] INTIALIZED WITH NULL VALUES TO MATCH MinorHI[1] WITH ARRAY INDEX[1]
		pMinorHI[0].idx 	= 0;
		pMinorHI[0].price = 0.00;

		// LOOP UNTIL FOUND REQUIRED NUMBER OF MinorHI VALUES
		int 	 count 	 = 0;
		double priceZZ = 0.00;

  	for(int i = mBarStart; i < mLookBackMax; i++) {

	    priceZZ = GetIndexZZ(bufferMinorHI,i);															// GET ZigZag[i] VALUE FOR bufferMinorHI
			if(priceZZ <= 0.00)			continue;																		// CONTINUE TO NEXT INDEX IN LOOP

	    if(count <= mMaxIndex) {    																				// ADD VALUE TO pMinorLO ARRAY
	      pMinorHI[count+1].idx 	= i;																			// ON FIRST MinorHI, count = 0 + 1 FOR INDEX[1]
	      pMinorHI[count+1].price = priceZZ;
	      count++;
	    }

	    if(count >= mMaxIndex)   return;           													// EXIT, IF REQUIRED mMaxIndex VALUES ARE FOUND
	  }

} // END Of method GetMinorHI()
//+-----------------------------------------------------------------------------------------------------------------------------+
//| METHOD:  			GetIndexZZ()
//| APPLICATION:  METHOD TO RETURN ZigZag VALUE FOR SPECIFIED BUFFER AND CHART INDEX
//+-----------------------------------------------------------------------------------------------------------------------------+
double CiZigZag::GetIndexZZ(int pBufferNo,int pIndex) {

  	double bufferData[1];																									// DEFINE LOCAL SINGLE ELEMENT STATIC BUFFER

	  if(CopyBuffer(mHandleZZ,pBufferNo,pIndex,1,bufferData) < 1) {
			ResetLastError();
	    PrintFormat("%s: Copy data Error[%i] iZigZag Buffer#[%i] Index[%i]",string(__FUNCTION__),GetLastError(),pBufferNo,pIndex);
	    return(0.0);
	  }

	  return(bufferData[0]);

} // END Of method GetIndexZZ()
//+-----------------------------------------------------------------------------------------------------------------------------+
